<?php
/**
  * Class used to manage static files cache and put together optimized scripts code
  */
  
class lcso_engine {
    const ENGINE_VERSION = '2.0'; // (string) scripts manipulation's code versioning 

    protected $subj 	= ''; // (string) subject (css or js)
	protected $src		= array(); // (array) script sources
    public $wp_scripts_baseurl 	= ''; // (string) wordpress baseurl for WP default scripts
    
	public $basedir 	= ''; // (string)cache folder's wrapper - basedir
	public $baseurl 	= ''; // (string) cache folder's wrapper - baseurl
	
    public $cache_folder_name = 'lcso-cache'; // (string) cache folder name
    public $cache_folder_path = ''; // (string) full cache folder path

    protected $cache_creation_times = array(); // (array) storing the caached files creation times for browser cache
    
	private $css_import = array(); // array containing imports to be prepended
    
    
    
	/* 
     * Setup variables
	 * 
	 * @param $subj (string)
	 * @param $src (array)
	 * @param $wp_scripts_baseurl (string) - wordpress baseurl for WP default scripts
	 */
	function __construct($subj, $src, $wp_scripts_baseurl, $basedir = '', $baseurl = '') {
		$this->subj = (in_array($subj, array('js', 'css'))) ? $subj : 'css';
		$this->src = (array)$src;
		$this->wp_scripts_baseurl = $wp_scripts_baseurl;

         
        // define basedir/url
        $wpul_dir = wp_upload_dir();
	
        $dir_arr = explode('/', $wpul_dir['basedir']);
        array_pop($dir_arr);
        $this->basedir = implode('/', $dir_arr);

        $url_arr = explode('/', $wpul_dir['baseurl']);
        array_pop($url_arr);
        $this->baseurl = implode('/', $url_arr);
        
		
        $this->cache_folder_path = $this->basedir .'/'. $this->cache_folder_name;
	}
    	
	
	
    
	/*
	 * cache folder existence check - creates it if not exists
	 * @return (bool) - if returns false means folder can't be created
	 */
	public function cache_folder_check($second_check = false) {
		if(isset($GLOBALS['lcso_cache_folder_checked'])) {
            return $GLOBALS['lcso_cache_folder_checked'];   
        }
        
        if(!file_exists($this->cache_folder_path)) {
			if($second_check) {
                $GLOBALS['lcso_cache_folder_checked'] = false;
                return false;
            }
			
			wp_mkdir_p($this->cache_folder_path);
			return $this->cache_folder_check(true);
		}	
		else {
            $GLOBALS['lcso_cache_folder_checked'] = true;
			return true;	
		}
	}
	
	
	
	
	/* can manage files within cache folder?
	 * @return (bool)
	 */
	public function files_write_permissions_check() {
        if(!$this->cache_folder_check()) {
            return false;   
        }
        if(isset($GLOBALS['lcso_files_write_permissions_checked'])) {
            return $GLOBALS['lcso_files_write_permissions_checked'];
        }
        
        require_once(ABSPATH .'wp-admin/includes/file.php');
        global $wp_filesystem;
        WP_Filesystem();
        
        $demo_file_path = $this->cache_folder_path .'/permisions_check.css';
        $response = $wp_filesystem->put_contents($demo_file_path, '1', FS_CHMOD_FILE);
        
		if(!$response) {
            $GLOBALS['lcso_files_write_permissions_checked'] = false;
            return false;
        }
        else {
            @unlink($demo_file_path);
            
            $GLOBALS['lcso_files_write_permissions_checked'] = true;
            return true;
        }
	}

    
	
	
	/* Are there cached scripts in the destination folder?
	 * @return (bool)
	 */
	public function has_cached_scripts() {
        if(!$this->cache_folder_check()) {
            return false;   
        }
		$files = (array)@glob($this->cache_folder_path .'/*');

        foreach($files as $file) { 
            if(is_file($file)) {
                return true;
            }
        }
        
        return false;
	}
	
	
	
    
    /* returns the URL attributes defining the optimizer preferences for the subject */
    private function subj_optim_prefs_atts() {
        global $lcso_optim_pref;
        $atts = '';
        
        if($this->subj == 'css') {
			if($lcso_optim_pref['no_css_min']) {$atts .= '&no_css_min';}
            if($lcso_optim_pref['no_css_import']) {$atts .= '&no_css_import';}
        }
        else {
            if($lcso_optim_pref['no_js_min']) {$atts .= '&no_js_min';}
        }
        return $atts;
    }
    
    
    
	
	/*
	 * Given minification parameters and involved scripts, returns the cache file name
	 * @return (string)
	 */
	protected function get_cache_filename() {
        global $lcso_optim_pref;
        
		$id = 'subj='. $this->subj .'&baseurl='. $this->wp_scripts_baseurl .'&src='. urlencode(implode('|', $this->src)) .'&engine_ver='. urlencode(self::ENGINE_VERSION) . $this->subj_optim_prefs_atts();
        
		return md5($id) .'.'. $this->subj;	
	}
	
	
	
	
	/* try creating the cached script
	 * @return (bool) - if returns false means folder can't be created
	 */
	protected function create_cache_file($code) {
        global $wp_filesystem;
        WP_Filesystem();
        
		// place @font-face them on top of the rest
		if($this->subj == 'css') {	
			preg_match_all('/@font-face\K[^}]*(?=})/', $code, $matches, PREG_SET_ORDER);
			if(is_array($matches)) {
				
				// remove existing ones
				preg_replace('/@font-face\K[^}]*(?=})/', '', $code); 
	
				foreach($matches as $m) {
					$code = '@font-face'. $m[0]. '}'. $code;	
				}
			}
		}
		
        // identify what is in the file
        $code = '/*
'.implode('
', $this->src) .'
*/

'. $code;
        
        
        $cache_filename = $this->get_cache_filename();
		$file_path = $this->cache_folder_path .'/'. $cache_filename;
        
        $this->cache_creation_times[ $cache_filename ] = gmdate('U');
		return ($wp_filesystem->put_contents($file_path, trim($code), FS_CHMOD_FILE)) ? true : false;
	}
	
    
	
    
    /* Given an optimized fileame, returns the URL attribute relateed to the creation date  */
    private function creation_date_url_attr($filename) {
        return (isset($this->cache_creation_times[$filename])) ? '?v='. $this->cache_creation_times[$filename] : '';    
    }
    
    
    
    
	/* 
     * Create optimized code and returns cache file URL
	 * @return (bool)
	 */
	public function get_optimized_url() {
        if(!$this->cache_folder_check() || !$this->files_write_permissions_check()) {
            return 'lcso-server-permissions-issues';   
        }
        
        $filename 	= $this->get_cache_filename();
        $file_path 	= $this->cache_folder_path .'/'. $filename;

        $cache_creation_times = (array)get_option('lcso_cache_times', array());
        $creation_date_param = (isset($cache_creation_times[$filename])) ? '?v='. $cache_creation_times[$filename] : '';

        $creation_date_url_attr = $this->creation_date_url_attr($filename);
        $url = $this->baseurl .'/'. $this->cache_folder_name .'/'. $filename . $creation_date_url_attr;

        // check for existing cached file
        if(file_exists($file_path) ) {
            return $url;	
        }

        // otherwise create code
        $code = $this->create_code(); 
        if(empty($code)) {
            return '';
        }

        $this->create_cache_file($code);

        if(empty($creation_date_url_attr)) {
            $url .= $this->creation_date_url_attr($filename);
            update_option('lcso_cache_times', $this->cache_creation_times, false);
        }
        
		return $url;
	}
    
    


	/* turn CSS relative paths to absolute, import @import files and move @font-face on top */
	private function css_rel2abs($src, $code) {
		if(empty($code)) {
            return '';
        }
        global $lcso_optim_pref;
		
		// get css file baseurl
		$arr = explode('?', $src);
		$src = $arr[0];
		
		$arr = explode('/', $src);
		array_pop($arr);
		$path = implode('/', $arr).'/';
		
		
		// replace relative uRLS
		$absoluteUrl = '((?:https?:)?//)';
		$rawData = '(?:data\:?:)';
		$relativeUrl = '\s*([\'"]?((' . $absoluteUrl . ')|(' . $rawData . ')))';
		$search = '#url\((?!' . $relativeUrl . ')\s*([\'"])?#';
		$replace = "url($6{$path}";
		
		$code = preg_replace($search, $replace, $code);	
		
		if(strpos(strtolower($src), 'https') !== false) {
			$code = str_replace('http://', 'https://', $code);	
		}
		

		// @import management
		if(preg_match_all('/(@import) (url)\(([^>]*?)\);/', $code, $matches)) {
			
			// complete @import inclusion
			if(!$lcso_optim_pref['no_css_import']){
				$found = array();
				
				foreach($matches[0] as $match) {
					$found[] = trim(str_replace( array("'", '"', ')', '(', 'url', '@import', ';'), '', $match));
				}
			  	
				$a = 0;
				foreach($found as $remote_url) {

					// replace // with http(s)://
					if(substr($remote_url, 0, 2) == '//') {
						$remote_url = (strpos(strtolower($remote_url), 'https') !== false) ? 'https:'.$remote_url : 'http:'.$remote_url;	
					}
					
					// strip URL parameters
					if(strpos($match, '?') !== false) {
						$raw = explode('?', $match);
						$match = $raw[0];	
					}
					
					// add
					$remote = '
			
/* @import src - '. $remote_url .' */

'. trim($this->css_rel2abs($remote_url, $this->lcso_curl($remote_url))); // recursive
					
					$code = str_replace($matches[0][$a], $remote, $code);
					$a++;
				}
			}
			
			// move @imports on top
			else {
				foreach($matches[0] as $row) {
					$orig_row = $row; 
					$code = str_replace($orig_row, '', $code);
					
					// replace // with http(s)://
					if(strpos(strtolower($row), 'http') === false && strpos(strtolower($row), 'https') === false) {
						$arr = explode('//', $row);
						$arr[0] = (strpos(strtolower($src), 'https') !== false) ? $arr[0].'https:' : $arr[0].'http:';
						$row = implode('//', $arr);
					}
					
					// add
					$this->css_import[] = $row;
					$code = str_replace($row, '', $code);
				}
			}
		}


		// avoid bad URLs
		$code = preg_replace('/wp-content\K[^.]*(?=wp-content)/', '', $code); 
		$code = str_replace('wp-contentwp-content', 'wp-content', $code); 
		
		return $code;
	}
	
    
	
	
	/* file_get_contents/cURL call */
	private function lcso_curl($url) {
		global $lcso_optim_pref;
        $subj = $this->subj;

		// local file - set to recall through file_get_contents - not for Fontawesome
		if(strpos(strtolower($url), site_url()) !== false) {
			$url = str_replace(site_url(), '', $url);
			
			if(substr($url, 0, 1) != '/') {
				$url = '/'. $url;
			}
		}


		// manage wp default scripts
		if(strpos(strtolower($url), 'http') === false) {
			$pos = strrpos($this->basedir, '/');
			$basepath = substr($this->basedir, 0, $pos);
			
			$url_arr = explode('?', $url);
			$code = (current_user_can('manage_options')) ? file_get_contents($basepath . $url_arr[0]) : @file_get_contents($basepath . $url_arr[0]);
		}

		else {

			// whether to use WP HTTP APIs or cURL
			if(function_exists('wp_remote_get')) {
				$args = array(
					'timeout'     => 3,
					'redirection' => 3,
				); 
				
				$response = wp_remote_get($url, $args);
				if(is_wp_error($response) || !isset($response['body'])) {
                    return '';
                }
				
				$data 		= wp_remote_retrieve_body($response);
				$mime 		= wp_remote_retrieve_header($response, 'content-type');
				$http_code 	= wp_remote_retrieve_response_code($response);
			}
			
			else {
				$ch = curl_init();
				
				curl_setopt($ch, CURLOPT_AUTOREFERER, true);
				curl_setopt($ch, CURLOPT_HEADER, 0);
				curl_setopt($ch, CURLOPT_USERAGENT, true);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
				curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
				curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
				curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
				curl_setopt($ch, CURLOPT_URL, $url);
				
				$data = curl_exec($ch);
				$mime = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
				$http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
				
				curl_close($ch);
			}
		
			// continue only if HTTP code is ok
			if($http_code != 200) {
				return '/* error '. $http_code .' */';
			}
	
	
			// be sure fetched data is congruous
			if($subj == 'js') {
				$code = (strpos($mime, 'text/javascript') !== false || strpos($mime, 'application/javascript') !== false) ? $data : '';	
			} else {
				$code = (strpos($mime, 'text/css') !== false) ? $data : '';	
			}
		}
		
		$code = trim($code);
		
		// minification and CSS import management
		if(!empty($code)) {
			if($subj == 'css') {
				if(!$lcso_optim_pref['no_css_min']) { 
                    require_once('minify.php');
					$min = new lcso_minify($code, $subj);
					$code = $min->get_css();
				}
				
				$code = $this->css_rel2abs($url, $code);
			}
			
			return trim($code);
		}
		
		return '';
	}

    
	

	/* optimized script creation
	 * @return (string) code block ready to be used
	 */
	 
	public function create_code() {
		$code = '';
		
		foreach($this->src as $src) {
			
			// JS
			if($this->subj == 'js') {			
				if(isset($_REQUEST['lcso_avoid'])) {
					$code .= '
				
/* src - '. $src .' */

'. trim($this->lcso_curl($src));	
				}
				else {
					$code .= trim( $this->strip_init_comment( $this->lcso_curl($src)));	
				}
			}
			
			
			// CSS
			else {
				if(isset($_REQUEST['lcso_avoid'])) {
					$code .= '
					
/* src - '. $src .' */

'. trim($this->css_rel2abs($src, $this->lcso_curl($src)));
				}
				else {
					$code .= trim( $this->strip_init_comment( $this->css_rel2abs($src, $this->lcso_curl($src))));	
				}
			}
		}
		
		
		// if is css - prepend imports
		if($this->subj == 'css' && !empty($this->css_import)) {
			$code = implode('
', $this->css_import). '
'. $code;
		}
		
		return trim($code);
	}
	
	
	
	
	/* remove the first comment starting with "/*" from a CSS or javascript file */
	private function strip_init_comment($code) {
		$trimmed = trim($code);
		
		if(substr($code, 0, 2) == '/*') {
			$comm_end_pos  = strpos($trimmed, '*/');
			
			if($comm_end_pos !== false) {
				$code = substr($trimmed, $comm_end_pos + 2);	
			}
		}
		return $code;	
	}

}