<?php
/* 
Plugin Name: LC Scripts Optimizer - by LCweb
Plugin URI: https://lcweb.it/
Description: Let's try optimizing scripts usage to speed-up your site!
Author: Luca Montanari (LCweb)
Version: 2.0.0
Author URI: https://lcweb.it/
Requires at least: 5.0
Requires PHP: 7.0
*/  




/////////////////////////////////////////////
/////// MAIN DEFINES ////////////////////////
/////////////////////////////////////////////

// plugin path
$wp_plugin_dir = substr(plugin_dir_path(__FILE__), 0, -1);
define('LCSO_DIR', $wp_plugin_dir);

// plugin url
$wp_plugin_url = substr(plugin_dir_url(__FILE__), 0, -1);
define('LCSO_URL', $wp_plugin_url);

// plugin version
define('LCSO_VER', '2.0.0');







/////////////////////////////////////////////
/////// FORCING DEBUG ///////////////////////
/////////////////////////////////////////////

if(isset($_REQUEST['lcso_php_debug'])) {
	ini_set('display_errors', 1);
	ini_set('display_startup_errors', 1);
	error_reporting(E_ALL);	
}





/////////////////////////////////////////////
/////// MULTILANGUAGE SUPPORT ///////////////
/////////////////////////////////////////////

function lcso_multilanguage() {
	if(!is_admin()) {
        return false;
    }
  
    $ml_key = 'lcso_ml';
    $basedir = LCSO_DIR;
    
    if(version_compare($GLOBALS['wp_version'], '6.7', '<')) {
        $param_array = explode('/', $basedir);
        load_plugin_textdomain($ml_key, false, end($param_array) .'/languages');
    }
    else {
        load_textdomain($ml_key, $basedir .'/languages/'. $ml_key .'-'. determine_locale() .'.mo');
    }  
}
add_action('init', 'lcso_multilanguage', 1);





/////////////////////////////////////////////
/////// MAIN SCRIPT & CSS INCLUDES //////////
/////////////////////////////////////////////

function lcso_scripts() { 
    global $current_screen;
    if(!$current_screen || strpos($current_screen->base, 'tools_page_lcso_settings') === false) {
        return;
    }
    	
    wp_enqueue_style('lcso_settings', LCSO_URL .'/settings/settings_style.css', 989, LCSO_VER);
	wp_enqueue_style('lcso_admin', LCSO_URL .'/css/admin.css', 999, LCSO_VER);
    
    wp_enqueue_script("jquery");
    wp_enqueue_script('lc-switch-v2', LCSO_URL .'/js/lc-switch/lc_switch.min.js', 200, '2.0.3', true);
    wp_enqueue_script('lc-wp-popup-message', LCSO_URL .'/js/lc-wp-popup-message/lc_wp_popup_message.min.js', 200, '1.2.1', true);
    
    wp_enqueue_style('lcwp-lc-select', LCSO_URL .'/js/lc-select/themes/lcwp_prefixed.css', 230, '1.1.8');
    wp_enqueue_script('lc-select', LCSO_URL .'/js/lc-select/lc_select.min.js', 1, '1.1.8', true);
}
add_action('admin_enqueue_scripts', 'lcso_scripts');





//////////////////////////////////////
///////// MENU ///////////////////////
//////////////////////////////////////

function lcso_settings_add_submenu() {	
    add_submenu_page('tools.php', 'LC Scripts Optimizer', 'LC Scripts Optimizer', 'install_plugins', 'lcso_settings', 'lcso_settings_include');
}
add_action('admin_menu', 'lcso_settings_add_submenu');

function lcso_settings_include() {
	include_once(LCSO_DIR .'/settings/view.php');	
}





///////////////////////////////////////
/// PLUGINS LIST - ADDITIONAL LINKS ///
///////////////////////////////////////

function lcso_plugins_list_links($links, $file) {
    if($file != 'lc-scripts-optimizer/lc_scripts_optimizer.php') {
        return $links;    
    }
    
    if(current_user_can('install_plugins')) {
        $links['lcso_goto_settings'] = '<a href="'. admin_url('tools.php?page=lcso_settings') .'">'. esc_html__('Manage settings', 'lcso_ml') .'</a>';
    }
    
    $links['lcso_check_lcweb_pjcts_link'] = '<a href="https://lcweb.it/products-overview/" target="_blank" class="lcso_check_lcweb_pjcts_link"><i class="dashicons dashicons-star-filled"></i>'. esc_html__('Check the other LCweb WordPress plugins!', 'lcso_ml') .'</a>';
    
    return $links;
}
add_filter('plugin_row_meta', 'lcso_plugins_list_links', 100, 2);





///////////////////////////////////////
/// GO TO SETTINGS ON ACTIVATION //////
///////////////////////////////////////

function lcso_redirect_on_activation($plugin) {
    if($plugin == 'lc-scripts-optimizer/lc_scripts_optimizer.php') {
        exit(
            wp_redirect(
                admin_url('tools.php?page=lcso_settings')
            )
        );
    }
}
add_action('activated_plugin', 'lcso_redirect_on_activation');






///////////////////////////////////////////////////////
// HTACCESS WEAK - ON PLUGIN ACTIVATION/DEACTIVATION //
///////////////////////////////////////////////////////


function lcso_ht_on_activation() {
    include_once(LCSO_DIR . '/classes/htaccess_tweaks.php');
    $lcso_ht = new lcso_htaccess_tweaks;
    
    if(get_option('lcso_tweak_htaccess')) {
        $lcso_ht->tweak_it();   
    }
}
register_activation_hook(__FILE__, 'lcso_ht_on_activation');


function lcso_ht_on_deactivation() {
    include_once(LCSO_DIR . '/classes/htaccess_tweaks.php');
    $lcso_ht = new lcso_htaccess_tweaks;
    
    if(get_option('lcso_tweak_htaccess')) {
        $lcso_ht->restore_it();   
    }
}
register_deactivation_hook(__FILE__, 'lcso_ht_on_deactivation');



    

//////////////////////////////////////////////
///////// MAIN INCLUDES //////////////////////
//////////////////////////////////////////////

// frontend actualization
include_once(LCSO_DIR .'/classes/frontend.php');


// HTML Minification
include_once(LCSO_DIR .'/classes/html_minifier.php');


// HTACCESS Tweaks
include_once(LCSO_DIR .'/classes/htaccess_tweaks.php');


// ajax operations
include_once(LCSO_DIR .'/ajax.php');

