<?php

// LCSO-FILTER - manipulate settings tabs
$tabs = array(
	'main_opts' 	   => esc_html__('Initialization', 'lcso_ml'),
	'scripts_filter'   => esc_html__('Scripts Filter', 'lcso_ml'),
	'ignored_scripts'  => esc_html__('Ignored Scripts', 'lcso_ml'),
    'support_the_pjct' => '🌟 '. esc_html__('Enjoying the plugin?', 'lcso_ml'),
);
$GLOBALS['lcso_settings_tabs'] = apply_filters('lcso_settings_tabs', $tabs);	




// system status
include_once(LCSO_DIR .'/classes/engine.php');
$engine = new lcso_engine('js', array(), '');

if($engine->cache_folder_check() && $engine->files_write_permissions_check()) {
    $system_status = '
    <div id="lcso_status_ok">
        <p>
            <strong>'. esc_html__("Everything's ok!", 'lcso_ml') .'</strong>';
    
            if($engine->has_cached_scripts()) {
                $system_status .= '
                <input type="button" class="button-secondary lcso_clean_cache" value="'. esc_html__('Clean cache', 'lcso_ml') .'" />';
            }
    
    $system_status .= '
        </p> 
        <p>'. esc_html__("Static files are properly stored in this folder", 'lcso_ml') .': <em>'. $engine->cache_folder_path .'</em></p>
    </div>';	
}
else {
    $system_status = '
    <div id="lcso_bad_status">
        <p><strong>'. esc_html__("Permission problems!", 'lcso_ml') .'</strong></p>
        <p>'. esc_html__("Your server has files creation restrictions and the plugin can't create this folder", 'lcso_ml') .': <em>'. $engine->cache_folder_path .'</em></p>
    </div>';
}




// STRUCTURE
/* tabs index => array( 
	'sect_id' => array(
		'sect_name'	=> name
		'fields'	=> array(
			...
		)
	)
   )
*/

$structure = array();

// MAIN OPTIONS
$structure['main_opts'] = array(
	
    'system_status' => array(
		'sect_name'	=> esc_html__('System Status', 'lcso_ml'),
		'fields' 	=> array(
			
            'lcso_system_status' => array(
				'type'      => 'message',
				'content'   => $system_status,
			),
        ),
    ),
    
    
	'init' => array(
		'sect_name'	=> esc_html__('Initialization', 'lcso_ml'),
		'fields' 	=> array(
			
            'lcso_enable' => array(
				'label' => __("Enable scripts optimization?", 'lcso_ml'),
				'type'	=> 'select',
				'val' 	=> array(
                    ''          => esc_html__('Do not manage scripts', 'lcso_ml'),
                    'dev_mode'  => esc_html__("Manage scripts for unlogged users", 'lcso_ml'),
                    'test_mode' => esc_html__("Manage scripts for WP admins (test mode)", 'lcso_ml'),
                    '1'         => esc_html__("Always manage scripts", 'lcso_ml'),
                ),
				'note'	=> esc_html__("Turn on the magic!", 'lcso_ml'), 
			), 
            'lcso_tweak_htaccess' => array(
				'label' => esc_html__('Use static files cache headers?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> esc_html__("Speeds up the website's rendering for people having already browser the pages", 'lcso_ml'),
			),
            
            
            /*'pg_redirect_page' => array(
				'type'		=> 'custom',
				'callback'	=> 'pc_redirect_custom_field',
				'validation'=> array(
					array('index' => 'pg_redirect_page', 'label' => __("Main target for redirect-restricted pages", 'lcso_ml'), 'required'=>true),
					array('index' => 'pg_redirect_page_custom', 'label' => __("Main target for redirect-restricted pages", 'lcso_ml') .' - '. __('Custom URL', 'lcso_ml'), 'type' => 'url'),
				)
			),*/
		),
	),
    
    
	'engine_tuning' => array(
		'sect_name'	=>  esc_html__('Engine Tuning', 'lcso_ml'),
		'fields' 	=> array(
            
            'lcso_not_for_js' => array(
				'label' => esc_html__('Avoid processing JavaScript files?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> '',
			),
            'lcso_not_for_css' => array(
				'label' => esc_html__('Avoid processing CSS files?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> '',
			),
            'lcso_not_for_html' => array(
				'label' => esc_html__('Avoid minifying HTML code?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> '',
			),
            'spcr1' => array(
				'type' => 'spacer',
			),
            'lcso_only_filter_js' => array(
				'label' => esc_html__('Do NOT concat Javascript?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> esc_html__('Check to disable JavaScript files concatenation', 'lcso_ml') .'.'. esc_html__('Should be checked only noticing incompatibilities', 'lcso_ml') .'.',
                
                'js_vis'=> array(
					'linked_field' 	=> 'lcso_not_for_js',
					'condition'		=> false,
				)
			),
            'lcso_only_filter_css' => array(
				'label' => esc_html__('Do NOT concat CSS?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> esc_html__('Check to disable CSS files concatenation', 'lcso_ml') .'.'. esc_html__('Should be checked only noticing incompatibilities', 'lcso_ml') .'.',
                
                'js_vis'=> array(
					'linked_field' 	=> 'lcso_not_for_css',
					'condition'		=> false,
				)
			),
            'spcr2' => array(
				'type' => 'spacer',
			),
            'lcso_no_js_min' => array(
				'label' => esc_html__('Do NOT minify JavaScript codes?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> esc_html__('Check to disable Javascript codes minification', 'lcso_ml') .'.'. esc_html__('Should be checked only noticing incompatibilities', 'lcso_ml') .'.',
                
                'js_vis'=> array(
					'linked_field' 	=> 'lcso_not_for_js',
					'condition'		=> false,
				)
			),
            'lcso_inl_js_ignore' => array(
				'label' => esc_html__('Do NOT minify inline JavaScript codes?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> esc_html__('Check to ignore Javascript codes found in the HTML', 'lcso_ml') .'.'. esc_html__('Should be checked only noticing incompatibilities', 'lcso_ml') .'.',
                
                'js_vis'=> array(
					'linked_field' 	=> 'lcso_not_for_html',
					'condition'		=> false,
				)
			),
            'lcso_no_css_min' => array(
				'label' => esc_html__('Do NOT minify CSS codes?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> esc_html__('Check to disable CSS codes minification', 'lcso_ml') .'.'. esc_html__('Should be checked only noticing incompatibilities', 'lcso_ml') .'.',
                
                'js_vis'=> array(
					'linked_field' 	=> 'lcso_not_for_css',
					'condition'		=> false,
				)
			),
            'lcso_no_css_import' => array(
				'label' => esc_html__('Do NOT process @import in CSS files?', 'lcso_ml'),
				'type'	=> 'checkbox',
				'note'	=> esc_html__('Should be checked only noticing incompatibilities', 'lcso_ml'),
                
                'js_vis'=> array(
					'linked_field' 	=> 'lcso_not_for_css',
					'condition'		=> false,
				)
			),
		),
	),
);




// SCRIPTS FILTER
$structure['scripts_filter'] = array(
	'scripts_filter_subj' => array(
		'sect_name'	=> false,
		'fields' 	=> array(
			
            'lcso_scripts_filter_subj' => array(
				'type'		=> 'custom',
				'callback'	=> 'lcso_scripts_filter_subj_cb',
			),
        ),
    ),
    
    'scripts_filter_list' => array(
		'sect_name'	=> false,
		'fields' 	=> array(
			
            'lcso_scripts_filter_list' => array(
				'type'		=> 'custom',
				'callback'	=> 'lcso_scripts_filter_list_cb',
                'validation'=> array(
					array('index' => 'lcso_js_ignore', 'label' => "ignored JS scripts"),
					array('index' => 'lcso_js_filter', 'label' => "filtered JS scripts"),
                    array('index' => 'lcso_css_ignore', 'label' => "ignored CSS scripts"),
					array('index' => 'lcso_css_filter', 'label' => "filtered CSS scripts"),
				)
			),
        ),
    ),
);




// IGNORED SCRIPTS
$structure['ignored_scripts'] = array(
	'main_reg_opts' => array(
		'sect_name'	=> esc_html__('Ignored Scripts', 'lcso_ml'),
		'fields' 	=> array(
			
            'lcso_ignored_js' => array(
				'label' => esc_html__('Any JavaScript file to ignore?', 'lcso_ml'),
				'type'	=> 'textarea',
				'placeh'=> '',
				'note'	=> esc_html__('One URL per line, supports regular expressions', 'lcso_ml'),
			),
            'lcso_ignored_css' => array(
				'label' => esc_html__('Any CSS file to ignore?', 'lcso_ml'),
				'type'	=> 'textarea',
				'placeh'=> '',
				'note'	=> esc_html__('One URL per line, supports regular expressions', 'lcso_ml'),
			),
        ),
    ),
);




// ENJOYING THE PLUGIN?
$structure['support_the_pjct'] = array(
	'main_reg_opts' => array(
		'sect_name'	=> false,
		'fields' 	=> array(
			
            'lcso_donate_if_like' => array(
				'type'		=> 'custom',
				'callback'	=> 'lcso_donate_if_like_cb',
			),
        ),
    ),
);







// LCSO-FILTER - manipulate settings structure
$GLOBALS['lcso_settings_structure'] = apply_filters('lcso_settings_structure', $structure);